@@ -1,7 +1,38 @@
 /*
- * Copyright (c) 2014-2015 LastPass.
+ * encrypted vault parsing
+ *
+ * Copyright (C) 2014-2016 LastPass.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
+ *
+ * See LICENSE.OpenSSL for more details regarding this exception.
  */
-
 #include "blob.h"
 #include "config.h"
 #include "endpoints.h"
@@ -31,12 +62,15 @@
 # endif
 #endif
 
+struct app *account_to_app(const struct account *account)
+{
+	return container_of(account, struct app, account);
+}
+
 void share_free(struct share *share)
 {
 	if (!share)
 		return;
-	if (--share->refcount > 0)
-		return;
 
 	free(share->name);
 	free(share->id);
@@ -44,16 +78,6 @@ void share_free(struct share *share)
 	free(share);
 }
 
-void share_assign(struct share *share, struct share **ptr)
-{
-	if (!share) {
-		*ptr = NULL;
-		return;
-	}
-	++share->refcount;
-	*ptr = share;
-}
-
 void field_free(struct field *field)
 {
 	if (!field)
@@ -66,10 +90,9 @@ void field_free(struct field *field)
 	free(field);
 }
 
-void account_free(struct account *account)
+void account_free_contents(struct account *account)
 {
-	if (!account)
-		return;
+	struct field *field, *tmp;
 
 	free(account->id);
 	free(account->name);
@@ -84,11 +107,63 @@ void account_free(struct account *account)
 	free(account->username_encrypted);
 	free(account->password_encrypted);
 	free(account->note_encrypted);
-	for (struct field *field = account->field_head, *next_field = NULL; field; field = next_field) {
-		next_field = field->next;
+
+	list_for_each_entry_safe(field, tmp, &account->field_head, list) {
 		field_free(field);
 	}
-	share_free(account->share);
+}
+
+void app_free(struct app *app)
+{
+	account_free_contents(&app->account);
+	free(app->appname);
+	free(app->extra);
+	free(app->extra_encrypted);
+	free(app->wintitle);
+	free(app->wininfo);
+	free(app->exeversion);
+	free(app->warnversion);
+	free(app->exehash);
+}
+
+bool account_is_group(struct account *account)
+{
+	return !strcmp(account->url, "http://group");
+}
+
+struct app *new_app()
+{
+	struct app *app = new0(struct app, 1);
+	struct account *account = &app->account;
+
+	app->appname = xstrdup("");
+	app->extra = xstrdup("");
+	app->extra_encrypted = xstrdup("");
+
+	INIT_LIST_HEAD(&account->field_head);
+	account->is_app = true;
+
+	return app;
+}
+
+struct account *new_account()
+{
+	struct account *account = new0(struct account, 1);
+	INIT_LIST_HEAD(&account->field_head);
+	return account;
+}
+
+void account_free(struct account *account)
+{
+	if (!account)
+		return;
+
+	if (account->is_app) {
+		app_free(account_to_app(account));
+		return;
+	}
+
+	account_free_contents(account);
 	free(account);
 }
 
@@ -97,26 +172,31 @@ void blob_free(struct blob *blob)
 	if (!blob)
 		return;
 
-	for (struct account *account = blob->account_head, *next_account = NULL; account; account = next_account) {
-		next_account = account->next;
+	struct account *account, *tmp;
+	struct share *share, *tmp_share;
+
+	list_for_each_entry_safe(account, tmp, &blob->account_head, list)
 		account_free(account);
-	}
+
+	list_for_each_entry_safe(share, tmp_share, &blob->share_head, list)
+		share_free(share);
+
 	free(blob);
 }
 
 struct blob_pos {
-	const char *data;
+	const unsigned char *data;
 	size_t len;
 };
 
 struct chunk {
 	char name[4 + 1];
-	const char *data;
+	const unsigned char *data;
 	size_t len;
 };
 
 struct item {
-	const char *data;
+	const unsigned char *data;
 	size_t len;
 };
 
@@ -176,7 +256,7 @@ static char *read_hex_string(struct chunk *chunk)
 	if (item.len == 0)
 		return xstrdup("");
 
-	result = hex_to_bytes(item.data, &str);
+	result = hex_to_bytes((char *) item.data, (unsigned char **) &str);
 	if (result) {
 		free(str);
 		return NULL;
@@ -194,7 +274,7 @@ static char *read_plain_string(struct chunk *chunk)
 	if (item.len == 0)
 		return xstrdup("");
 
-	return xstrndup(item.data, item.len);
+	return xstrndup((char *) item.data, item.len);
 }
 
 static char *read_crypt_string(struct chunk *chunk, const unsigned char key[KDF_HASH_LEN], char **stored_base64)
@@ -230,30 +310,33 @@ static int read_boolean(struct chunk *chunk)
 	return item.data[0] == '1';
 }
 
-#define entry_plain(var) do { \
+#define entry_plain_at(base, var) do { \
 	char *__entry_val__ = read_plain_string(chunk); \
 	if (!__entry_val__) \
 		goto error; \
-	parsed->var = __entry_val__; \
+	base->var = __entry_val__; \
 	} while (0)
-#define entry_hex(var) do { \
+#define entry_plain(var) entry_plain_at(parsed, var)
+#define entry_hex_at(base, var) do { \
 	char *__entry_val__ = read_hex_string(chunk); \
 	if (!__entry_val__) \
 		goto error; \
-	parsed->var = __entry_val__; \
+	base->var = __entry_val__; \
 	} while (0)
+#define entry_hex(var) entry_hex_at(parsed, var)
 #define entry_boolean(var) do { \
 	int __entry_val__ = read_boolean(chunk); \
 	if (__entry_val__ < 0) \
 		goto error; \
 	parsed->var = __entry_val__; \
 	} while (0)
-#define entry_crypt(var) do { \
-	char *__entry_val__ = read_crypt_string(chunk, key, &parsed->var##_encrypted); \
+#define entry_crypt_at(base, var) do { \
+	char *__entry_val__ = read_crypt_string(chunk, key, &base->var##_encrypted); \
 	if (!__entry_val__) \
 		goto error; \
-	parsed->var = __entry_val__; \
+	base->var = __entry_val__; \
 	} while (0)
+#define entry_crypt(var) entry_crypt_at(parsed, var)
 #define skip(placeholder) do { \
 	struct item skip_item; \
 	if (!read_item(chunk, &skip_item)) \
@@ -262,21 +345,21 @@ static int read_boolean(struct chunk *chunk)
 
 static struct account *account_parse(struct chunk *chunk, const unsigned char key[KDF_HASH_LEN])
 {
-	struct account *parsed = new0(struct account, 1);
+	struct account *parsed = new_account();
 
 	entry_plain(id);
 	entry_crypt(name);
 	entry_crypt(group);
 	entry_hex(url);
 	entry_crypt(note);
-	skip(fav);
+	entry_boolean(fav);
 	skip(sharedfromaid);
 	entry_crypt(username);
 	entry_crypt(password);
 	entry_boolean(pwprotect);
 	skip(genpw);
 	skip(sn);
-	skip(last_touch);
+	entry_plain(last_touch);
 	skip(autologin);
 	skip(never_autofill);
 	skip(realm_data);
@@ -295,7 +378,7 @@ static struct account *account_parse(struct chunk *chunk, const unsigned char ke
 	skip(individualshare);
 	skip(notetype);
 	skip(noalert);
-	skip(last_modified_gmt);
+	entry_plain(last_modified_gmt);
 	skip(hasbeenshared);
 	skip(last_pwchange_gmt);
 	skip(created_gmt);
@@ -305,7 +388,10 @@ static struct account *account_parse(struct chunk *chunk, const unsigned char ke
 		parsed->name[0] = '\0';
 	if (parsed->group[0] == 16)
 		parsed->group[0] = '\0';
-	if (strlen(parsed->name) && strlen(parsed->group))
+
+	/* use name as 'fullname' only if there's no assigned group */
+	if (strlen(parsed->group) &&
+	    (strlen(parsed->name) || account_is_group(parsed)))
 		xasprintf(&parsed->fullname, "%s/%s", parsed->group, parsed->name);
 	else
 		parsed->fullname = xstrdup(parsed->name);
@@ -336,11 +422,25 @@ error:
 	return NULL;
 }
 
+static struct field *app_field_parse(struct chunk *chunk, const unsigned char key[KDF_HASH_LEN])
+{
+	struct field *parsed = new0(struct field, 1);
+
+	entry_plain(name);
+	entry_crypt(value);
+	entry_plain(type);
+
+	return parsed;
+error:
+	field_free(parsed);
+	return NULL;
+}
+
 static struct share *share_parse(struct chunk *chunk, const struct private_key *private_key)
 {
 	struct share *parsed = new0(struct share, 1);
 	struct item item;
-	_cleanup_free_ char *ciphertext = NULL;
+	_cleanup_free_ unsigned char *ciphertext = NULL;
 	_cleanup_free_ char *hex_key = NULL;
 	_cleanup_free_ unsigned char *key = NULL;
 	_cleanup_free_ char *base64_name = NULL;
@@ -359,7 +459,7 @@ static struct share *share_parse(struct chunk *chunk, const struct private_key *
 
 	if (!read_item(chunk, &item) || item.len == 0 || item.len % 2 != 0)
 		goto error;
-	hex_to_bytes(item.data, &ciphertext);
+	hex_to_bytes((char *) item.data, &ciphertext);
 	hex_key = cipher_rsa_decrypt(ciphertext, item.len / 2, private_key);
 	if (!hex_key)
 		goto error;
@@ -369,7 +469,7 @@ static struct share *share_parse(struct chunk *chunk, const struct private_key *
 	len /= 2;
 	if (len != KDF_HASH_LEN)
 		goto error;
-	hex_to_bytes(hex_key, (char **)&key);
+	hex_to_bytes(hex_key, &key);
 	mlock(parsed->key, KDF_HASH_LEN);
 	memcpy(parsed->key, key, KDF_HASH_LEN);
 
@@ -387,67 +487,122 @@ error:
 	return NULL;
 }
 
+static struct app *app_parse(struct chunk *chunk, const unsigned char key[KDF_HASH_LEN])
+{
+	struct app *app = new_app();
+	struct account *parsed = &app->account;
+
+	entry_plain(id);
+	entry_hex_at(app, appname);
+	entry_crypt_at(app, extra);
+	entry_crypt(name);
+	entry_crypt(group);
+	entry_plain(last_touch);
+	skip(fiid);
+	entry_boolean(pwprotect);
+	entry_boolean(fav);
+	entry_plain_at(app, wintitle);
+	entry_plain_at(app, wininfo);
+	entry_plain_at(app, exeversion);
+	skip(autologin);
+	entry_plain_at(app, warnversion);
+	entry_plain_at(app, exehash);
+
+	parsed->username = xstrdup("");
+	parsed->password = xstrdup("");
+	parsed->note = xstrdup("");
+	parsed->url = xstrdup("");
+
+	if (strlen(parsed->group) &&
+	    (strlen(parsed->name) || account_is_group(parsed)))
+		xasprintf(&parsed->fullname, "%s/%s", parsed->group, parsed->name);
+	else
+		parsed->fullname = xstrdup(parsed->name);
+
+	return app;
+error:
+	app_free(app);
+	return NULL;
+}
+
 #undef entry_plain
+#undef entry_plain_at
 #undef entry_hex
 #undef entry_boolean
 #undef entry_crypt
+#undef entry_crypt_at
 #undef skip
 
-struct blob *blob_parse(const char *blob, size_t len, const unsigned char key[KDF_HASH_LEN], const struct private_key *private_key)
+struct blob *blob_parse(const unsigned char *blob, size_t len, const unsigned char key[KDF_HASH_LEN], const struct private_key *private_key)
 {
 	struct blob_pos blob_pos = { .data = blob, .len = len };
 	struct chunk chunk;
-	struct account *account, **next_account;
-	struct field *field, **next_field = NULL;
-	struct share *last_share = NULL;
+	struct account *account = NULL;
+	struct field *field;
+	struct share *share, *last_share = NULL;
+	struct app *app = NULL;
 	struct blob *parsed;
 	_cleanup_free_ char *versionstr = NULL;
 
 	parsed = new0(struct blob, 1);
 	parsed->local_version = false;
-	next_account = &parsed->account_head;
+	INIT_LIST_HEAD(&parsed->account_head);
+	INIT_LIST_HEAD(&parsed->share_head);
 
 	while (read_chunk(&blob_pos, &chunk)) {
 		if (!strcmp(chunk.name, "LPAV")) {
-			versionstr = xstrndup(chunk.data, chunk.len);
+			versionstr = xstrndup((char *) chunk.data, chunk.len);
 			parsed->version = strtoull(versionstr, NULL, 10);
 		} else if (!strcmp(chunk.name, "ACCT")) {
 			account = account_parse(&chunk, last_share ? last_share->key : key);
 			if (!account)
 				goto error;
-			share_assign(last_share, &account->share);
 
-			*next_account = account;
-			next_account = &account->next;
+			if (last_share) {
+				account->share = last_share;
+				char *tmp = account->fullname;
+				xasprintf(&account->fullname, "%s/%s",
+					  last_share->name, tmp);
+				free(tmp);
+			}
+
+			list_add(&account->list, &parsed->account_head);
 
-			next_field = &account->field_head;
 		} else if (!strcmp(chunk.name, "ACFL") || !strcmp(chunk.name, "ACOF")) {
-			if (!next_field)
+			if (!account)
 				goto error;
 
 			field = field_parse(&chunk, last_share ? last_share->key : key);
 			if (!field)
 				goto error;
 
-			*next_field = field;
-			next_field = &field->next;
-		} else if (!strcmp(chunk.name, "LOCL"))
+			list_add_tail(&field->list, &account->field_head);
+		} else if (!strcmp(chunk.name, "LOCL")) {
 			parsed->local_version = true;
-		else if (!strcmp(chunk.name, "SHAR")) {
-			share_free(last_share);
-			share_assign(share_parse(&chunk, private_key), &last_share);
+		} else if (!strcmp(chunk.name, "SHAR")) {
+			share = share_parse(&chunk, private_key);
+			last_share = share;
+			if (share)
+				list_add_tail(&share->list, &parsed->share_head);
+		} else if (!strcmp(chunk.name, "AACT")) {
+			app = app_parse(&chunk, last_share ? last_share->key : key);
+			if (app)
+				list_add_tail(&app->account.list, &parsed->account_head);
+		} else if (!strcmp(chunk.name, "AACF")) {
+			if (!app)
+				goto error;
+			field = app_field_parse(&chunk, last_share ? last_share->key : key);
+			if (!field)
+				goto error;
+			list_add_tail(&field->list, &app->account.field_head);
 		}
 	}
 
 	if (!versionstr)
 		goto error;
-	if (!parsed->account_head)
-		goto error;
-	share_free(last_share);
 	return parsed;
 
 error:
-	share_free(last_share);
 	blob_free(parsed);
 	return NULL;
 }
@@ -482,18 +637,20 @@ static void write_plain_string(struct buffer *buffer, char *bytes)
 static void write_hex_string(struct buffer *buffer, char *bytes)
 {
 	_cleanup_free_ char *hex = NULL;
-	bytes_to_hex(bytes, &hex, strlen(bytes));
+	bytes_to_hex((unsigned char *) bytes, &hex, strlen(bytes));
 	write_plain_string(buffer, hex);
 }
-static void write_crypt_string(struct buffer *buffer, char *bytes, const unsigned char key[KDF_HASH_LEN])
+static void write_crypt_string(struct buffer *buffer, char *enc_str)
 {
-	_cleanup_free_ char *encrypted = NULL;
+	_cleanup_free_ unsigned char *encrypted = NULL;
 	size_t len;
 
-	len = cipher_aes_encrypt(bytes, key, &encrypted);
-	if (!encrypted)
-		die("Could not write encrypted string.");
-	write_item(buffer, encrypted, len);
+	/*
+	 * enc_str is base64-encoded, but we write out raw bytes in
+	 * the saved blob, so un-base64.
+	 */
+	len = cipher_unbase64(enc_str, &encrypted);
+	write_item(buffer, (char *) encrypted, len);
 }
 
 static void write_boolean(struct buffer *buffer, bool yes)
@@ -509,20 +666,63 @@ static void write_chunk(struct buffer *dstbuffer, struct buffer *srcbuffer, char
 	write_item(dstbuffer, srcbuffer->bytes, srcbuffer->len);
 }
 
-static void write_account_chunk(struct buffer *buffer, struct account *account, const unsigned char key[KDF_HASH_LEN])
+static void write_app_chunk(struct buffer *buffer, struct account *account)
 {
 	struct buffer accbuf, fieldbuf;
+	struct field *field;
+	struct app *app = account_to_app(account);
 
 	memset(&accbuf, 0, sizeof(accbuf));
 	write_plain_string(&accbuf, account->id);
-	write_crypt_string(&accbuf, account->name, key);
-	write_crypt_string(&accbuf, account->group, key);
+	write_hex_string(&accbuf, app->appname);
+	write_crypt_string(&accbuf, app->extra_encrypted);
+	write_crypt_string(&accbuf, account->name_encrypted);
+	write_crypt_string(&accbuf, account->group_encrypted);
+	write_plain_string(&accbuf, "skipped");
+	write_plain_string(&accbuf, "skipped");
+	write_boolean(&accbuf, account->pwprotect);
+	write_boolean(&accbuf, account->fav);
+	write_plain_string(&accbuf, "skipped");
+	write_plain_string(&accbuf, "skipped");
+	write_plain_string(&accbuf, "skipped");
+	write_plain_string(&accbuf, "skipped");
+	write_plain_string(&accbuf, "skipped");
+	write_plain_string(&accbuf, "skipped");
+	write_chunk(buffer, &accbuf, "AACT");
+	free(accbuf.bytes);
+	list_for_each_entry(field, &account->field_head, list) {
+		memset(&fieldbuf, 0, sizeof(fieldbuf));
+		write_plain_string(&fieldbuf, field->name);
+		if (!strcmp(field->type, "email") || !strcmp(field->type, "tel") || !strcmp(field->type, "text") || !strcmp(field->type, "password") || !strcmp(field->type, "textarea"))
+			write_crypt_string(&fieldbuf, field->value_encrypted);
+		else
+			write_plain_string(&fieldbuf, field->value);
+		write_plain_string(&fieldbuf, field->type);
+		write_chunk(buffer, &fieldbuf, "AACF");
+		free(fieldbuf.bytes);
+	}
+}
+
+static void write_account_chunk(struct buffer *buffer, struct account *account)
+{
+	struct buffer accbuf, fieldbuf;
+	struct field *field;
+
+	if (account->is_app) {
+		write_app_chunk(buffer, account);
+		return;
+	}
+
+	memset(&accbuf, 0, sizeof(accbuf));
+	write_plain_string(&accbuf, account->id);
+	write_crypt_string(&accbuf, account->name_encrypted);
+	write_crypt_string(&accbuf, account->group_encrypted);
 	write_hex_string(&accbuf, account->url);
-	write_crypt_string(&accbuf, account->note, key);
+	write_crypt_string(&accbuf, account->note_encrypted);
 	write_plain_string(&accbuf, "skipped");
 	write_plain_string(&accbuf, "skipped");
-	write_crypt_string(&accbuf, account->username, key);
-	write_crypt_string(&accbuf, account->password, key);
+	write_crypt_string(&accbuf, account->username_encrypted);
+	write_crypt_string(&accbuf, account->password_encrypted);
 	write_boolean(&accbuf, account->pwprotect);
 	write_plain_string(&accbuf, "skipped");
 	write_plain_string(&accbuf, "skipped");
@@ -552,12 +752,12 @@ static void write_account_chunk(struct buffer *buffer, struct account *account,
 	write_plain_string(&accbuf, "skipped");
 	write_chunk(buffer, &accbuf, "ACCT");
 	free(accbuf.bytes);
-	for (struct field *field = account->field_head; field; field = field->next) {
+	list_for_each_entry(field, &account->field_head, list) {
 		memset(&fieldbuf, 0, sizeof(fieldbuf));
 		write_plain_string(&fieldbuf, field->name);
 		write_plain_string(&fieldbuf, field->type);
 		if (!strcmp(field->type, "email") || !strcmp(field->type, "tel") || !strcmp(field->type, "text") || !strcmp(field->type, "password") || !strcmp(field->type, "textarea"))
-			write_crypt_string(&fieldbuf, field->value, key);
+			write_crypt_string(&fieldbuf, field->value_encrypted);
 		else
 			write_plain_string(&fieldbuf, field->value);
 		write_boolean(&fieldbuf, field->checked);
@@ -575,28 +775,29 @@ size_t blob_write(const struct blob *blob, const unsigned char key[KDF_HASH_LEN]
 {
 	struct buffer buffer;
 	struct share *last_share = NULL;
-	_cleanup_free_ char *version;
+	struct account *account;
+	UNUSED(key);
 
 	memset(&buffer, 0, sizeof(buffer));
 
-	version = xultostr(blob->version);
+	_cleanup_free_ char *version = xultostr(blob->version);
 	buffer_append(&buffer, "LPAV", 4);
 	write_plain_string(&buffer, version);
 	buffer_append(&buffer, "LOCL", 4);
 	write_plain_string(&buffer, LASTPASS_CLI_VERSION);
 
-	for (struct account *account = blob->account_head; account; account = account->next) {
+	list_for_each_entry(account, &blob->account_head, list) {
 		if (!account->share)
-			write_account_chunk(&buffer, account, key);
+			write_account_chunk(&buffer, account);
 	}
-	for (struct account *account = blob->account_head; account; account = account->next) {
+	list_for_each_entry(account, &blob->account_head, list) {
 		if (!account->share)
 			continue;
 		if (last_share != account->share) {
 			write_share_chunk(&buffer, account->share);
 			last_share = account->share;
 		}
-		write_account_chunk(&buffer, account, account->share->key);
+		write_account_chunk(&buffer, account);
 	}
 
 	*out = buffer.bytes;
@@ -605,7 +806,7 @@ size_t blob_write(const struct blob *blob, const unsigned char key[KDF_HASH_LEN]
 
 static struct blob *local_blob(const unsigned char key[KDF_HASH_LEN], const struct private_key *private_key)
 {
-	_cleanup_free_ char *blob = NULL;
+	_cleanup_free_ unsigned char *blob = NULL;
 	size_t len = config_read_encrypted_buffer("blob", &blob, key);
 	if (!blob)
 		return NULL;
@@ -673,28 +874,20 @@ void blob_save(const struct blob *blob, const unsigned char key[KDF_HASH_LEN])
 	config_write_encrypted_buffer("blob", bluffer, len, key);
 }
 
-static char *encrypt_and_base64(const char *str, unsigned const char key[KDF_HASH_LEN])
-{
-	char *intermediate = NULL;
-	char *base64 = NULL;
-	size_t len;
-
-	base64 = trim(xstrdup(str));
-	len = cipher_aes_encrypt(base64, key, &intermediate);
-	free(base64);
-	base64 = cipher_base64(intermediate, len);
-	free(intermediate);
-	return base64;
-}
-
 #define set_field(obj, field) do { \
 	free(obj->field); \
 	obj->field = field; \
 } while (0)
 #define set_encrypted_field(obj, field) do { \
-	set_field(obj, field); \
+	if (!obj->field || !field || strcmp(obj->field, field)) { \
+		set_field(obj, field); \
+		free(obj->field##_encrypted); \
+		obj->field##_encrypted = encrypt_and_base64(field, account->share ? account->share->key : key); \
+	} \
+} while (0)
+#define reencrypt_field(obj, field) do { \
 	free(obj->field##_encrypted); \
-	obj->field##_encrypted = encrypt_and_base64(field, account->share ? account->share->key : key); \
+	obj->field##_encrypted = encrypt_and_base64(obj->field, account->share ? account->share->key : key); \
 } while (0)
 
 void account_set_username(struct account *account, char *username, unsigned const char key[KDF_HASH_LEN])
@@ -717,6 +910,21 @@ void account_set_note(struct account *account, char *note, unsigned const char k
 {
 	set_encrypted_field(account, note);
 }
+void account_set_url(struct account *account, char *url, unsigned const char key[KDF_HASH_LEN])
+{
+	UNUSED(key);
+	set_field(account, url);
+}
+void account_set_appname(struct account *account, char *appname, unsigned const char key[KDF_HASH_LEN])
+{
+	UNUSED(key);
+	struct app *app;
+	if (!account->is_app)
+		return;
+
+	app = account_to_app(account);
+	set_field(app, appname);
+}
 void field_set_value(struct account *account, struct field *field, char *value, unsigned const char key[KDF_HASH_LEN])
 {
 	if (!strcmp(field->type, "email") || !strcmp(field->type, "tel") || !strcmp(field->type, "text") || !strcmp(field->type, "password") || !strcmp(field->type, "textarea"))
@@ -724,20 +932,118 @@ void field_set_value(struct account *account, struct field *field, char *value,
 	else
 		set_field(field, value);
 }
-void account_set_fullname(struct account *account, char *fullname, unsigned const char key[KDF_HASH_LEN])
+
+static bool is_shared_folder_name(const char *fullname)
 {
-	char *slash = strrchr(fullname, '/');
+	return !strncmp(fullname, "Shared-", 7) && strchr(fullname, '/');
+}
+
+void account_reencrypt(struct account *account, const unsigned char key[KDF_HASH_LEN])
+{
+	struct field *field;
+
+	reencrypt_field(account, name);
+	reencrypt_field(account, group);
+	reencrypt_field(account, username);
+	reencrypt_field(account, password);
+	reencrypt_field(account, note);
+
+	list_for_each_entry(field, &account->field_head, list) {
+		reencrypt_field(field, value);
+	}
+}
+
+/*
+ * Set just group and name, assuming we've stripped off any leading
+ * shared folder from fullname.
+ */
+static void account_set_group_name(struct account *account,
+				   const char *groupname,
+				   unsigned const char key[KDF_HASH_LEN])
+{
+	char *slash = strrchr(groupname, '/');
 	if (!slash) {
-		account_set_name(account, xstrdup(fullname), key);
+		account_set_name(account, xstrdup(groupname), key);
 		account_set_group(account, xstrdup(""), key);
 	} else {
 		account_set_name(account, xstrdup(slash + 1), key);
-		account_set_group(account, xstrndup(fullname, slash - fullname), key);
+		account_set_group(account, xstrndup(groupname, slash - groupname), key);
 	}
+}
+
+void account_set_fullname(struct account *account, char *fullname, unsigned const char key[KDF_HASH_LEN])
+{
+	char *groupname = fullname;
+
+	/* skip Shared-XXX/ for shared folders */
+	if (is_shared_folder_name(fullname)) {
+		char *tmp = strchr(fullname, '/');
+		if (tmp)
+			groupname = tmp + 1;
+	}
+	account_set_group_name(account, groupname, key);
 	free(account->fullname);
 	account->fullname = fullname;
 }
 
+struct share *find_unique_share(struct blob *blob, const char *name)
+{
+       struct share *share;
+
+       list_for_each_entry(share, &blob->share_head, list) {
+               if (!strcasecmp(share->name, name)) {
+                       return share;
+               }
+       }
+       return NULL;
+}
+
+/*
+ * Assign an account to the proper shared folder, if any.
+ *
+ * If the share changed from whatever it was previously, the account
+ * fields are reencrypted with either the share key or the blob key.
+ *
+ * This function may exit if the name represents a shared folder but
+ * same folder is not available.
+ */
+void account_assign_share(struct blob *blob, struct account *account,
+			  unsigned const char key[KDF_HASH_LEN])
+{
+	struct share *share, *old_share;
+	_cleanup_free_ char *shared_name = NULL;
+	char *name = account->fullname;
+
+	old_share = account->share;
+
+	/* strip off shared groupname */
+	char *slash = strchr(name, '/');
+	if (!slash) {
+		account->share = NULL;
+		goto reencrypt;
+	}
+
+	shared_name = xstrndup(name, slash - name);
+
+	/* find a share matching group name */
+	share = find_unique_share(blob, shared_name);
+
+	if (!share && is_shared_folder_name(name)) {
+		/* don't allow normal folders named like SFs */
+		die("Unable to find shared folder for %s in blob\n", name);
+	}
+
+	account->share = share;
+
+	/* update group name to not include new share, if needed */
+	if (share)
+		account_set_group_name(account, slash + 1, key);
+
+reencrypt:
+	if (old_share != account->share)
+		account_reencrypt(account, key);
+}
+
 struct account *notes_expand(struct account *acc)
 {
 	struct account *expand;
@@ -749,14 +1055,14 @@ struct account *notes_expand(struct account *acc)
 	if (strcmp(acc->url, "http://sn"))
 		return NULL;
 
-	expand = new0(struct account, 1);
+	expand = new_account();
 
 	expand->id = xstrdup(acc->id);
 	expand->pwprotect = acc->pwprotect;
 	expand->name = xstrdup(acc->name);
 	expand->group = xstrdup(acc->group);
 	expand->fullname = xstrdup(acc->fullname);
-	share_assign(acc->share, &expand->share);
+	expand->share = acc->share;
 
 	if (strncmp(acc->note, "NoteType:", 9))
 		return NULL;
@@ -791,8 +1097,7 @@ struct account *notes_expand(struct account *acc)
 			field->type = xstrdup("text");
 			field->name = xstrdup(name);
 			field->value = xstrdup(value);
-			field->next = expand->field_head;
-			expand->field_head = field;
+			list_add(&field->list, &expand->field_head);
 		}
 skip:
 		free(line);
@@ -803,7 +1108,7 @@ skip:
 		if (!*start)
 			break;
 	}
-	if (!expand->note && !expand->username && !expand->url && !expand->password && !expand->field_head)
+	if (!expand->note && !expand->username && !expand->url && !expand->password && list_empty(&expand->field_head))
 		expand->note = xstrdup(acc->note);
 	else if (!expand->note)
 		expand->note = xstrdup("");
@@ -819,8 +1124,9 @@ skip:
 struct account *notes_collapse(struct account *acc)
 {
 	struct account *collapse;
+	struct field *field;
 
-	collapse = new0(struct account, 1);
+	collapse = new_account();
 
 	collapse->id = xstrdup(acc->id);
 	collapse->pwprotect = acc->pwprotect;
@@ -831,9 +1137,9 @@ struct account *notes_collapse(struct account *acc)
 	collapse->username = xstrdup("");
 	collapse->password = xstrdup("");
 	collapse->note = xstrdup("");
-	share_assign(acc->share, &collapse->share);
+	collapse->share = acc->share;
 
-	for (struct field *field = acc->field_head; field; field = field->next) {
+	list_for_each_entry(field, &acc->field_head, list) {
 		trim(field->value);
 		trim(field->name);
 		if (!strcmp(field->name, "NoteType"))
