@@ -71,6 +71,7 @@ int cmd_edit(int argc, char **argv)
 	enum blobsync sync = BLOB_SYNC_AUTO;
 	struct account *editable;
 	enum edit_choice choice = EDIT_ANY;
+	enum note_type note_type = NOTE_TYPE_NONE;
 
 	#define ensure_choice() if (choice != EDIT_ANY) goto choice_die;
 	while ((option = getopt_long(argc, argv, "up", long_options, &option_index)) != -1) {
@@ -128,7 +129,8 @@ int cmd_edit(int argc, char **argv)
 	editable = find_unique_account(blob, name);
 	if (!editable)
 		return edit_new_account(session, blob, sync, name, choice,
-					field, non_interactive, key);
+					field, non_interactive, false,
+					note_type, key);
 
 	if (editable->share && editable->share->readonly)
 		die("%s is a readonly shared entry from %s. It cannot be edited.",
