@@ -1,9 +1,38 @@
 /*
- * Copyright (c) 2014 LastPass.
+ * command for generating passwords
  *
+ * Copyright (C) 2014-2016 LastPass.
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
+ *
+ * See LICENSE.OpenSSL for more details regarding this exception.
  */
-
 #include "cmd.h"
 #include "util.h"
 #include "config.h"
@@ -105,17 +134,16 @@ int cmd_generate(int argc, char **argv)
 			account_free(notes_collapsed);
 		}
 	} else {
-		new = new0(struct account, 1);
+		new = new_account();
 		new->id = xstrdup("0");
-
 		account_set_password(new, xstrdup(password), key);
 		account_set_fullname(new, xstrdup(name), key);
 		account_set_username(new, username ? username : xstrdup(""), key);
 		account_set_note(new, xstrdup(""), key);
 		new->url = url ? url : xstrdup("");
+		account_assign_share(blob, new, key);
 
-		new->next = blob->account_head;
-		blob->account_head = new;
+		list_add(&new->list, &blob->account_head);
 	}
 
 	lastpass_update_account(sync, key, session, found ? found : new, blob);
