@@ -1,5 +1,37 @@
 /*
- * Copyright (c) 2014-2015 LastPass.
+ * general utility functions used by multiple commands
+ *
+ * Copyright (C) 2014-2016 LastPass.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
+ *
+ * See LICENSE.OpenSSL for more details regarding this exception.
  */
 #include "cmd.h"
 #include "agent.h"
@@ -35,6 +67,28 @@ enum color_mode parse_color_mode_string(const char *colormode)
 		die_usage("... --color=auto|never|always");
 }
 
+bool parse_bool_arg_string(const char *extra)
+{
+	return !extra || strcmp(extra, "true") == 0;
+}
+
+enum note_type parse_note_type_string(const char *extra)
+{
+	enum note_type result;
+
+	result = notes_get_type_by_shortname(extra);
+	if (result == NOTE_TYPE_NONE) {
+		_cleanup_free_ char *params = NULL;
+		_cleanup_free_ char *usage = NULL;
+
+		params = note_type_usage();
+		xasprintf(&usage, "... %s", params);
+		die_usage(usage);
+	}
+
+	return result;
+}
+
 void init_all(enum blobsync sync, unsigned char key[KDF_HASH_LEN], struct session **session, struct blob **blob)
 {
 	if (!agent_get_decryption_key(key))
@@ -67,19 +121,31 @@ static int cmp_substr(const char *haystack, const char *needle)
 	return strstr(haystack, needle) == NULL;
 }
 
-static void search_accounts(struct blob *blob,
+/*
+ * Search accounts with a given comparator.
+ *
+ * Any matched account is removed from the accounts list, and added to
+ * ret_list.
+ *
+ * Note, the account list is iterated through match_list, so the caller
+ * must first create a list of possible matches (from blob->account_head).
+ * This is done instead of searching blob->account_head directly to enable
+ * multiple searches of the potential match set.
+ */
+static void search_accounts(struct list_head *accounts,
 			    const void *needle,
 			    int (*cmp)(const char *haystack, const char *needle),
 			    int fields,
 			    struct list_head *ret_list)
 {
-	for (struct account *account = blob->account_head; account;
-	     account = account->next) {
+	struct account *account, *tmp;
+	list_for_each_entry_safe(account, tmp, accounts, match_list) {
 		if (((fields & ACCOUNT_ID) && cmp(account->id, needle) == 0) ||
 		    ((fields & ACCOUNT_NAME) && cmp(account->name, needle) == 0) ||
 		    ((fields & ACCOUNT_FULLNAME) && cmp(account->fullname, needle) == 0) ||
 		    ((fields & ACCOUNT_URL) && cmp(account->url, needle) == 0) ||
 		    ((fields & ACCOUNT_USERNAME) && cmp(account->username, needle) == 0)) {
+			list_del(&account->match_list);
 			list_add_tail(&account->match_list, ret_list);
 		}
 	}
@@ -91,14 +157,14 @@ static void search_accounts(struct blob *blob,
  * @pattern - a basic regular expression
  * @fields - which fields to search on
  */
-void find_matching_regex(struct blob *blob, const char *pattern,
+void find_matching_regex(struct list_head *accounts, const char *pattern,
 			 int fields, struct list_head *ret_list)
 {
 	regex_t regex;
 
 	if (regcomp(&regex, pattern, REG_ICASE))
 		die("Invalid regex '%s'", pattern);
-	search_accounts(blob, &regex, cmp_regex, fields, ret_list);
+	search_accounts(accounts, &regex, cmp_regex, fields, ret_list);
 	regfree(&regex);
 }
 
@@ -109,25 +175,27 @@ void find_matching_regex(struct blob *blob, const char *pattern,
  * @pattern - a basic regular expression
  * @fields - which fields to search on
  */
-void find_matching_substr(struct blob *blob, const char *pattern,
+void find_matching_substr(struct list_head *accounts, const char *pattern,
 			  int fields, struct list_head *ret_list)
 {
-	search_accounts(blob, pattern, cmp_substr, fields, ret_list);
+	search_accounts(accounts, pattern, cmp_substr, fields, ret_list);
 }
 
 /*
- * Search blob for any and all accounts matching a given name.
+ * Search list of accounts for any and all accounts matching a given name.
  * Matching accounts are appended to ret_list which should be initialized
  * by the caller.
  *
  * In the case of an id match, we return only the matching id entry.
  */
-void find_matching_accounts(struct blob *blob, const char *name,
+void find_matching_accounts(struct list_head *accounts, const char *name,
 			    struct list_head *ret_list)
 {
 	/* look for exact id match */
-	for (struct account *account = blob->account_head; account; account = account->next) {
+	struct account *account;
+	list_for_each_entry(account, accounts, match_list) {
 		if (strcmp(name, "0") && !strcasecmp(account->id, name)) {
+			list_del(&account->match_list);
 			list_add_tail(&account->match_list, ret_list);
 			/* if id match, stop processing */
 			return;
@@ -135,18 +203,24 @@ void find_matching_accounts(struct blob *blob, const char *name,
 	}
 
 	/* search for fullname or name match */
-	search_accounts(blob, name, strcmp, ACCOUNT_NAME | ACCOUNT_FULLNAME,
+	search_accounts(accounts, name, strcmp,
+			ACCOUNT_NAME | ACCOUNT_FULLNAME,
 			ret_list);
 }
 
 struct account *find_unique_account(struct blob *blob, const char *name)
 {
 	struct list_head matches;
+	struct list_head potential_set;
 	struct account *account, *last_account;
 
 	INIT_LIST_HEAD(&matches);
+	INIT_LIST_HEAD(&potential_set);
 
-	find_matching_accounts(blob, name, &matches);
+	list_for_each_entry(account, &blob->account_head, list)
+		list_add(&account->match_list, &potential_set);
+
+	find_matching_accounts(&potential_set, name, &matches);
 
 	if (list_empty(&matches))
 		return NULL;
