@@ -4,6 +4,10 @@
 #include "kdf.h"
 #include <stdbool.h>
 
+struct public_key {
+	unsigned char *key;
+	size_t len;
+};
 struct private_key {
 	unsigned char *key;
 	size_t len;
@@ -12,6 +16,7 @@ struct session {
 	char *uid;
 	char *sessionid;
 	char *token;
+	char *server;
 	struct private_key private_key;
 };
 
@@ -21,5 +26,6 @@ bool session_is_valid(struct session *session);
 struct session *sesssion_load(unsigned const char key[KDF_HASH_LEN]);
 void session_save(struct session *session, unsigned const char key[KDF_HASH_LEN]);
 void session_set_private_key(struct session *session, unsigned const char key[KDF_HASH_LEN], const char *key_hex);
+void session_kill();
 
 #endif
