@@ -39,6 +39,7 @@
 #include "util.h"
 #include "config.h"
 #include "kdf.h"
+#include "log.h"
 #include "process.h"
 #include "password.h"
 #include "endpoints.h"
@@ -110,8 +111,9 @@ static void upload_queue_cleanup_failures()
 	while ((entry = readdir(dir))) {
 		_cleanup_free_ char *fn = NULL;
 
-		if (entry->d_type != DT_REG)
+		if (entry->d_type != DT_REG && entry->d_type != DT_UNKNOWN)
 			continue;
+
 		for (p = entry->d_name; *p; ++p) {
 			if (!isdigit(*p))
 				break;
@@ -137,6 +139,9 @@ static void upload_queue_drop(const char *name)
 	_cleanup_free_ char *old_full = NULL;
 	_cleanup_free_ char *new_full = NULL;
 	char *basename;
+	int ret;
+
+	lpass_log(LOG_DEBUG, "UQ: dropping %s\n", name);
 
 	make_upload_dir("upload-fail");
 
@@ -150,7 +155,9 @@ static void upload_queue_drop(const char *name)
 
 	old_full = config_path(name);
 	new_full = config_path(newname);
-	rename(old_full, new_full);
+	ret = rename(old_full, new_full);
+
+	lpass_log(LOG_DEBUG, "UQ: rename returned %d (errno=%d)\n", ret, errno);
 
 	upload_queue_cleanup_failures();
 }
@@ -169,8 +176,9 @@ static char *upload_queue_next_entry(unsigned const char key[KDF_HASH_LEN], char
 	if (!dir)
 		return NULL;
 	while ((entry = readdir(dir))) {
-		if (entry->d_type != DT_REG)
+		if (entry->d_type != DT_REG && entry->d_type != DT_UNKNOWN)
 			continue;
+
 		for (p = entry->d_name; *p; ++p) {
 			if (!isdigit(*p))
 				break;
@@ -217,6 +225,7 @@ static char *upload_queue_next_entry(unsigned const char key[KDF_HASH_LEN], char
 	result = config_read_encrypted_string(*name, key);
 	if (!result) {
 		/* could not decrypt: drop this file */
+		lpass_log(LOG_DEBUG, "UQ: unable to decrypt job %s\n", *name);
 		upload_queue_drop(*name);
 		config_unlink(*lock);
 		return NULL;
@@ -245,6 +254,9 @@ static void upload_queue_upload_all(const struct session *session, unsigned cons
 	int backoff;
 
 	while ((entry = upload_queue_next_entry(key, &name, &lock))) {
+
+		lpass_log(LOG_DEBUG, "UQ: processing job %s\n", name);
+
 		size = 0;
 		for (p = entry; *p; ++p) {
 			if (*p == '\n')
@@ -275,15 +287,22 @@ static void upload_queue_upload_all(const struct session *session, unsigned cons
 		backoff = 1;
 		for (int i = 0; i < 5; ++i) {
 			if (i) {
+				lpass_log(LOG_DEBUG, "UQ: attempt %d, sleeping %d seconds\n", i+1, backoff);
 				sleep(backoff);
 				backoff *= 8;
 			}
 
-			result = http_post_lastpass_v_noexit(argv[0],
-				session->sessionid, NULL, &argv[1],
+			lpass_log(LOG_DEBUG, "UQ: posting to %s\n", argv[0]);
+
+			result = http_post_lastpass_v_noexit(session->server, argv[0],
+				session, NULL, &argv[1],
 				&curl_ret, &http_code);
 
-			http_failed_all &= curl_ret == HTTP_ERROR_CODE;
+			http_failed_all &=
+				(curl_ret == HTTP_ERROR_CODE ||
+				 curl_ret == HTTP_ERROR_CONNECT);
+
+			lpass_log(LOG_DEBUG, "UQ: result %d (http_code=%ld)\n", curl_ret, http_code);
 
 			if (result && strlen(result))
 				should_fetch_new_blob_after = true;
@@ -292,12 +311,15 @@ static void upload_queue_upload_all(const struct session *session, unsigned cons
 				break;
 		}
 		if (!result) {
+			lpass_log(LOG_DEBUG, "UQ: failed, http_failed_all: %d\n", http_failed_all);
+
 			/* server failed response 5 times, remove it */
 			if (http_failed_all)
 				upload_queue_drop(name);
 
 			config_unlink(lock);
 		} else {
+			lpass_log(LOG_DEBUG, "UQ: succeeded\n");
 			config_unlink(name);
 			config_unlink(lock);
 		}
@@ -321,13 +343,17 @@ static void upload_queue_run(const struct session *session, unsigned const char
 	pid_t child = fork();
 	if (child < 0)
 		die_errno("fork(agent)");
+
 	if (child == 0) {
 		int null = open("/dev/null", 0);
+		int upload_log = null;
+
 		if (null >= 0) {
 			dup2(null, 0);
 			dup2(null, 1);
 			dup2(null, 2);
 			close(null);
+			close(upload_log);
 		}
 		setsid();
 		IGNORE_RESULT(chdir("/"));
@@ -337,7 +363,16 @@ static void upload_queue_run(const struct session *session, unsigned const char
 		signal(SIGQUIT, upload_queue_cleanup);
 		signal(SIGTERM, upload_queue_cleanup);
 		signal(SIGALRM, upload_queue_cleanup);
+		setvbuf(stdout, NULL, _IOLBF, 0);
+
+		if (http_init()) {
+			lpass_log(LOG_ERROR, "UQ: unable to restart curl\n");
+			_exit(EXIT_FAILURE);
+		}
+
+		lpass_log(LOG_DEBUG, "UQ: starting queue run\n");
 		upload_queue_upload_all(session, key);
+		lpass_log(LOG_DEBUG, "UQ: queue run complete\n");
 		upload_queue_cleanup(0);
 		_exit(EXIT_SUCCESS);
 	}
@@ -373,23 +408,21 @@ bool upload_queue_is_running(void)
 	return process_is_same_executable(pid);
 }
 
-void upload_queue_enqueue(enum blobsync sync, unsigned const char key[KDF_HASH_LEN], const struct session *session, const char *page, ...)
+void upload_queue_enqueue(enum blobsync sync, unsigned const char key[KDF_HASH_LEN], const struct session *session, const char *page, struct http_param_set *params)
 {
 	_cleanup_free_ char *sum = xstrdup(page);
 	char *next = NULL;
 	char *escaped = NULL;
-	va_list params;
 	char *param;
+	char **argv = params->argv;
 
-	va_start(params, page);
-	while ((param = va_arg(params, char *))) {
+	while ((param = *argv++)) {
 		escaped = pinentry_escape(param);
 		xasprintf(&next, "%s\n%s", sum, escaped);
 		free(escaped);
 		free(sum);
 		sum = next;
 	}
-	va_end(params);
 
 	upload_queue_write_entry(sum, key);
 
